# -*- encoding: utf-8 -*-

import httpx

from django.conf import settings
from django.utils import timezone
from asgiref.sync import sync_to_async
from django.shortcuts import redirect
from django.http import JsonResponse, HttpResponse
from django.views.generic import TemplateView, View

from apps.home.models import Server
from apps.authentication.models import DiscordUser, ServerMember


class IndexView(TemplateView):
    """
    View for the Index page.
    """

    template_name = "home/index.html"


class GuildsView(View):

    async def get(self, request, *args, **kwargs):
        if not await self.is_user_authenticated(request.user):
            return redirect("/oauth2/login")

        access_token = await self.get_user_access_token(request.user)

        async with httpx.AsyncClient() as client:
            response = await client.get(
                url=f"{settings.DISCORD_API_URL}/users/@me/guilds",
                headers={"Authorization": f"Bearer {access_token}"}
            )
            guild_data = response.json()


        # guilds where the user either administrates or owns
        cleaned_guild_data = []

        for item in guild_data:
            cleaned_data = await self.setup_server(request.user, item)
            if cleaned_data:
                cleaned_guild_data.append(cleaned_data)

        return JsonResponse(cleaned_guild_data, safe=False)


    @staticmethod
    @sync_to_async
    def get_user_access_token(user: DiscordUser) -> str:
        return user.access_token

    @staticmethod
    @sync_to_async
    def is_user_authenticated(user: DiscordUser) -> bool:
        return user.is_authenticated

    async def setup_server(self, user: DiscordUser, data: dict):
        is_owner = data["owner"]
        permissions = data["permissions"]
        admin_perm = 1 << 3

        # Ignore servers where the user isn't an administrator or owner
        if not ((int(permissions) & admin_perm) == admin_perm or is_owner):
            await self.delete_member(user, data["id"])
            return

        server = await Server.objects.aupdate_or_create(
            id=data["id"],
            defaults={
                "name": data["name"],
                "icon_hash": data["icon"]
            }
        )
    
        await ServerMember.objects.aupdate_or_create(
            user=user,
            server=server[0],
            defaults={
                "permissions": permissions,
                "is_owner": is_owner
            }
        )

        return data

    @staticmethod
    async def delete_member(user: DiscordUser, server_id: int):
        try:
            member = await ServerMember.objects.aget(user=user, server_id=server_id)
            await member.adelete()
        except ServerMember.DoesNotExist:
            pass
